package killer_sokoban;

import java.io.*;
import java.util.*;
import java.util.Map.Entry;


import javafx.application.Application;
import javafx.event.EventHandler;
import javafx.scene.Group;
import javafx.scene.Scene;
import javafx.scene.canvas.Canvas;
import javafx.scene.input.KeyCode;
import javafx.scene.input.KeyEvent;
import javafx.stage.Stage;



public class Main extends Application {
	
	class moveHandler implements EventHandler<KeyEvent>
	{

		@Override
		public void handle(KeyEvent ke) {
			//ha játék vége, nem lépünk..
			if (!gameRunning) return;
			
			
			if (ke.getCode().equals(KeyCode.UP))
				Main.workers.get("w1").step(Direction.UP);
			else if (ke.getCode().equals(KeyCode.RIGHT))
				Main.workers.get("w1").step(Direction.RIGHT);
			else if (ke.getCode().equals(KeyCode.DOWN))
				Main.workers.get("w1").step(Direction.DOWN);
			else if (ke.getCode().equals(KeyCode.LEFT))
				Main.workers.get("w1").step(Direction.LEFT);			
			else if (ke.getCode().equals(KeyCode.H))
				Main.workers.get("w1").putHoney();		
			else if (ke.getCode().equals(KeyCode.O))
				Main.workers.get("w1").putOil();
			
			view.update();
		}
		
	}
	
	private static HashMap<String, AbstractField> map = new HashMap<>(); //itt tároljuk a pályán lévő mezőket név (pl. "f_2_4") és referencia szerint
	private static HashMap<String, Worker> workers = new HashMap<>(); //a munkások
	private static HashMap<String, Crate> crates = new HashMap<>(); //a ládák - azért szedtem így külön mindent, hogy a játék vége detektálásánál végig lehessen kérdezgetni csak a ládákat - comment by SF
	private static boolean gameRunning;
	
	//A beolvasáshoz kellenek. Azért tettem ide őket, hogy a parseMap függvény is lássa őket.
	private static InputStreamReader isr =	new InputStreamReader(System.in);
	private static BufferedReader br = new BufferedReader(isr);
	
	private static View view=new View();
	
	@Override
	public void start(Stage primaryStage) throws Exception {
		// Ablak felépítése
		primaryStage.setTitle("My super killer sokoban");
        Group root = new Group();
        Canvas canvas = new Canvas(300, 250);
        view.setCanvas(canvas);
        root.getChildren().add(canvas);
        primaryStage.setScene(new Scene(root));
        primaryStage.show();
        
        //Event handlerek és egyebek
        primaryStage.addEventHandler(KeyEvent.KEY_PRESSED, new moveHandler());
        
        //Itt kellene betölteni a pályát
		runCommand("script testmap1.in");
		
		//Ez végzi a rajzolást
		view.update();
		
		//Egyéb adminisztratív teendők..
		gameRunning=true;
		
		
	}
	public static void drawRequiredFor(int y, int x)
	{
		AbstractField af=map.get("f_"+y+"_"+x);
		af.shouldDraw(view);
	}
	public static void main(String[] args) throws IOException {
		
		launch(args);
		
//		while (true) {
//			String line;
//			line = br.readLine();
//
//			if (line == null) break;
//			
//			runCommand(line);			
//		}
	}
	
	
	public static String getFieldName(AbstractField field)
	{
		return getKeyByValue(map, field);
	}
	
	public static String getMoveableName(Moveable m)
	{
		HashMap<String, Moveable> moveables = new HashMap<>(workers);
		moveables.putAll(crates);
		
		return getKeyByValue(moveables, m);
	}
	
	public static void deleteWorker(Worker w)
	{
		workers.remove(getKeyByValue(workers, w));
	}
	
	public static void deleteCrate(Crate c)
	{
		crates.remove(getKeyByValue(crates, c));
	}
	
	private static void endGame()
	{
		for (Entry<String, Worker> e : workers.entrySet())
		{
			System.out.println(e.getKey() + ": " + e.getValue());
		}
		gameRunning=false;
	}
	private static void runCommand(String cmdline) throws NumberFormatException, IOException
	{
		//Ha üres sort kaptunk, ignoráljuk
		//Emiatt áttekinthetőbb bemeneteket lehet csinálni
		if (cmdline.equals("")) return;
		
		
		//Parancs részekgre tördelése
		String cmd[];
		cmd = cmdline.split(" ");
		
		
		//Parancs értelmezés, és a szükséges függvény meghívása
		if (cmd[0].equals("parsemap")) {			
			parseMap(Integer.parseInt(cmd[1]), Integer.parseInt(cmd[2]));
		}
		else if (cmd[0].equals("worker")) {
			spawnWorker(cmd[1],Double.parseDouble(cmd[2]),cmd[3]);
		}
		else if (cmd[0].equals("crate")) {
			spawnCrate(cmd[1],cmd[2]);
		}
		else if (cmd[0].equals("setswitch")) {
			setSwitch(cmd[1], cmd[2]);
		}
		else if (cmd[0].equals("putfluid")) {
			putFluid(cmd[1], cmd[2].charAt(0));
		}
		else if (cmd[0].equals("step")) {
			step(cmd[1],cmd[2]);
		}
		else if (cmd[0].equals("stat")) {
			stat(cmd[1]);
		}
		else if (cmd[0].equals("script")) {
			script(cmd[1]);
		}
		else 
		{
			System.out.println("unrecognized command");
		}
		
	}
	
	private static void parseMap(int lines, int columns) throws IOException {
		//Tisztítás
		map.clear();
		workers.clear();
		crates.clear();
		
		//Feldolgozás
		//Mezők létrehozása
		for (int i=0; i<lines; i++)
		{
			String line= br.readLine();
			for (int j=0; j<columns; j++)
			{
				String name="f_"+i+"_"+j; //név generálása
				switch (line.charAt(j)) {
					case 'w' :
						map.put(name, new Wall());
						break;
					case 'm' :
						map.put(name, new SimpleField());
						break;
					case 'h' :
						map.put(name, new Hole());
						break;
					case 'k' :
						map.put(name, new SwitchField());
						break;
					case 's' :
						map.put(name, new SwitchableHole());
						break;
					case 't' :
						map.put(name, new TargetField());
						break;
				}
			}
		}
		
		
		//Szomszédságok beállítása
		//Minden mezőn végigmegyünk. Ha létezik a megfelelő irányú szomszédja, beállítjuk.
		for (int i=0; i<lines; i++)
		{
			for (int j=0; j<columns; j++)
			{
				String name="f_"+i+"_"+j; //név generálása
				String nameUp="f_"+(i-1)+"_"+j; //név generálása
				String nameRight="f_"+i+"_"+(j+1); //név generálása
				String nameDown="f_"+(i+1)+"_"+j; //név generálása
				String nameLeft="f_"+i+"_"+(j-1); //név generálása
				
				if (map.containsKey(nameUp))
					map.get(name).setNeighbour(Direction.UP, map.get(nameUp));
				if (map.containsKey(nameRight))
					map.get(name).setNeighbour(Direction.RIGHT, map.get(nameRight));
				if (map.containsKey(nameDown))
					map.get(name).setNeighbour(Direction.DOWN, map.get(nameDown));
				if (map.containsKey(nameLeft))
					map.get(name).setNeighbour(Direction.LEFT, map.get(nameLeft));
			}
		}
		
		//View újrainicializálása
		view.init(lines,  columns);
	}
	
	private static void spawnWorker(String name, double force, String field)
	{
		Worker w=new Worker(force);
		workers.put(name, w);
		AbstractField f=map.get(field);
		f.accept(Direction.RIGHT, w, w);
	}
	
	private static void spawnCrate(String name, String field)
	{
		Crate c=new Crate();
		crates.put(name, c);
		AbstractField f=map.get(field);
		f.accept(Direction.RIGHT, new Worker(0), c);
	}
	
	private static void setSwitch(String shole, String mySwitch)
	{
		((SwitchField) map.get(mySwitch)).setSHole((SwitchableHole) map.get(shole));
	}
	
	private static void putFluid(String field, char fluid)
	{
		SimpleField sf=((SimpleField) map.get(field));
		if (fluid=='h')
			sf.putHoney();
		else if (fluid=='o')
			sf.putOil();
		else
			System.out.println("you can only choose either 'h' or 'o'");
	}
	
	private static void step(String worker, String toward)
	{
		Worker w= (Worker) workers.get(worker);
		w.step(Direction.fromString(toward));
	}
	
	private static void stat(String object)
	{
		if (map.containsKey(object))
		{
			map.get(object).printStat();
		}
		else if (workers.containsKey(object))
		{
			workers.get(object).printStat();
		}
		else if (crates.containsKey(object))
		{
			crates.get(object).printStat();
		}
		else
		{
			System.out.println("This doesn't exist!");
		}
	}
	
	private static void script(String file) throws IOException
	{
		BufferedReader backup = br;
		
		//Maybe this works...
		File scriptFile=new File(file);
		FileInputStream fis=new FileInputStream(scriptFile);
		InputStreamReader isr =	new InputStreamReader(fis);
		br = new BufferedReader(isr);
		
		while (true) {
			String line;
			line = br.readLine();

			if (line == null) break;
			
			runCommand(line);			
		}
		
		br.close();
		
		br = backup;
		  
	}
	
	//code copied from StackOverFlow
	private static <T, E> T getKeyByValue(Map<T, E> map, E value) {
	    for (Entry<T, E> entry : map.entrySet()) {
	        if (Objects.equals(value, entry.getValue())) {
	            return entry.getKey();
	        }
	    }
	    return null;
	}

}
