package killer_sokoban;

public class SwitchableHole extends SimpleField {
		
	boolean opened=false;
	
	/**Ez a függvény azt jelzi a mező számára, hogy szeretnének rálépni.
	 * @param toward melyik irányból szeretnének rálépni
	 * @param w	munkás, aki a tolást indította
	 * @param m	mozgatható dolog a pályán, ez szeretne a mezőre lépni
	 * @return ha befogadja a paraméterül kapott Moveable-t, true-val tér vissza, ellenkező esetben false-szal.
	 */
	@Override
	public boolean accept(Direction toward, Worker w, Moveable m) {

		boolean ret=true;
		
		if (!opened)
		{
			ret=super.accept(toward, w, m);
		}
		
		//the hole could open while super.accept was being called
		//so it can't be an else branch
		if (opened)
		{			
			m.move(this);
			m.kill();
			ret=true;
		}
					

		return ret;
	}
	
	/**
	 * Kinyitja a lyukat.
	 */
	public void open()
	{
		if (myMoveable!=null)
			myMoveable.kill();
		opened=true;
	}
	
	/**
	 * Bezárja a lyukat.
	 */
	public void close()
	{
		opened=false;
	}
	
	public void printStat()
	{
		super.printStat();
		System.out.println("opened: " + opened);
	}
	
	@Override
	public void shouldDraw(View view)
	{
		//Ha nyitva van, rajzolunk egy lyukat is
		if (opened)
			view.drawHole();		
		
		//Akár nyitva van, akár nem, kirajajzoljuk a többit is.
		super.shouldDraw(view);
	}
	
}

