package killer_sokoban;


public class Worker extends Moveable {
	
	private boolean alive=true;
	private int points;
	private double force;
	private double activeForce;
	
	/**
	 * Konstruktor
	 */
	public Worker(double _force) {
		force = _force;
	}
	
	/**
	 * Lekéri a játékos pontjait, az eredmény megjelenítéséhez kell
	 * @return A pontok
	 */
	public int getPoints()
	{
		return points;
	}
	
	/**
	 * Ha a játékos szeretné léptetni valamely irányba a munkást, akkor hívódik meg ez a függvény..
	 * @param toward amelyik irányba szeretné léptetni a munkást.
	 * @return ha a lépés sikeres volt, igazzal, ellenkező esetben hamissal tér vissza.
	 */
	public boolean step(Direction toward)
	{
		activeForce = force;
		boolean result=myField.stepMe(toward, this);

		return result;
	}
	
	/**
	 * A játékos olajat helyez el a mezőjén.
	 */
	public void putOil()
	{
		if (myField!=null)
			myField.putOil();
	}
	
	/**
	 * A játékos mézet helyez el a mezőjén.
	 */
	public void putHoney()
	{
		if (myField!=null)
			myField.putHoney();
	}
	
	/**
	 * Pontot ad a játékosnak.
	 */
	public void incrementPoint()	{
		points++;
	}
	
	/**
	 * Pontot vesz el a játékostól.
	 */
	public void decrementPoint()	{
		points--;
	}
	
	/**
	 * A munkás meghal, és nem vesz részt a további játékban.
	 */
	public void kill() {
		force=0;
		alive=false;
		super.kill();
	}
	
	/**
	 * A munkás összenyomódásra kényszerül.
	 */
	protected boolean squeez()
	{
		kill();
		return true;
	}
	
	/**
	 * A munkás eldönti, hogy az újabb ládát még el tudja-e tolni
	 */
	public boolean decrementForce(double by) {		
		activeForce -= by;
		return (activeForce>=0.0);
	}
	
	public void printStat()
	{
		super.printStat();
		System.out.println("points: " + points);
		System.out.println("force: " + force);
		System.out.println("alive: " + alive);
	}
	
	/**
	 * A munkás kikapcsolja a kapcsolót, amire lépett
	 */
	public void onSwitch(SwitchField s) {
		s.turnOff();
	}
	
	@Override
	public void shouldDraw(View view)
	{
		super.shouldDraw(view);
		view.drawWorker();
	}

}
